package GMB::Plugin::Export;
use strict;
use warnings;
use constant
{	PID	=> 'Export',
	name	=> _"Export",
};

$::Plugins{&PID}=__PACKAGE__;

my %menuentry=
(topath =>
 {	label => _"Copy to portable player",	#label of the menu entry
	code => \&Copy,				#when menu entry selected
	test => sub {my $p=$::Options{'PLUGIN_'.PID.'_path'}; $p && (-d $p || $p=~m/%/);},	#the menu entry is displayed if returns true
	notempty => 'IDs',			#display only if at least one song
 },
 tom3u =>
 {	label => _"Export to .m3u file",
	code => \&ToM3U,
	notempty => 'IDs',
 },
 tocmd =>
 {	label => sub { ($::Options{'PLUGIN_'.PID.'_tocmd_label'} || _"Unamed custom command") },
	code => \&RunCommand,
	test => sub {my $c=$::Options{'PLUGIN_'.PID.'_tocmd_cmd'}; defined $c && $c ne '';},
	notempty => 'IDs',
 }
);

my %FLmenuentry=
(topath =>
 {	label => _"Copy to portable player",
	code => \&Copy,
	test => sub {my $p=$::Options{'PLUGIN_'.PID.'_path'}; $p && (-d $p || $p=~m/%/);},
	isdefined => 'filter',
 },
 tom3u =>
 {	label => _"Export to .m3u file",
	code => \&ToM3U,
	isdefined => 'filter',
 },
);

sub init
{	updatemenu();
}
sub end
{	updatemenu('remove');
}

sub prefbox
{	my $vbox=Gtk2::VBox->new(::FALSE, 2);
	my $label1=Gtk2::Label->new(_"Export plugin");
	my $label2=Gtk2::Label->new(_"Adds menu entries to song contextual menu");
	my $sg1=Gtk2::SizeGroup->new('horizontal');
	my $sg2=Gtk2::SizeGroup->new('horizontal');

	my $entry1=::NewPrefFileEntry('PLUGIN_'.PID.'_path',_"Player mounted on :",'folder',undef,undef,$sg1,$sg2);
	my $entry2=::NewPrefEntry('PLUGIN_'.PID.'_folderformat',_"Folder format :",undef,$sg1,$sg2,undef,_("These fields can be used :")."\n".::MakeReplaceText('talydnAY'));
	my $entry3=::NewPrefEntry('PLUGIN_'.PID.'_filenameformat',_"Filename format :",undef,$sg1,$sg2,undef,_("These fields can be used :")."\n".::MakeReplaceText('talydnAYo'));
	my $check1=::NewPrefCheckButton('PLUGIN_'.PID.'_topath',_"Copy to mounted portable player",sub { updatemenu(); },undef, ::Vpack($entry1,$entry2,$entry3) );

	my $entry4=::NewPrefEntry('PLUGIN_'.PID.'_tocmd_label',_"Menu entry name",undef,$sg1,$sg2,undef,_("Name under which the command will appear in the menu"));
	my $entry5=::NewPrefEntry('PLUGIN_'.PID.'_tocmd_cmd',_"System command :",undef,$sg1,$sg2,undef,_("These fields can be used :")."\n".::MakeReplaceText('ftalydnAY')."\n"._("In this case one command by file will be run\n\n".'Or you can use the field $files which will be replaced by the list of files, and only one command will be run'));
	my $check2=::NewPrefCheckButton('PLUGIN_'.PID.'_tocmd',_"Execute custom command on selected files",sub { updatemenu(); },undef, ::Vpack($entry4,$entry5) );

	my $check3=::NewPrefCheckButton('PLUGIN_'.PID.'_tom3u',_"Export to .m3u file",sub { updatemenu(); });
	$vbox->pack_start($_,::FALSE,::FALSE,2) for $label1,$label2,$check1,$check2,$check3;
	return $vbox;
}

sub updatemenu
{	my $removeall=$_[0];
	for my $eid (keys %menuentry)
	{	my $menu=\@::SongCMenu;
		my $entry=$menuentry{$eid};
		if (!$removeall && $::Options{'PLUGIN_'.PID.'_'.$eid})
		{	push @$menu,$entry unless (grep $_==$entry, @$menu);
		}
		else
		{	@$menu =grep $_!=$entry, @$menu;
		}
	}
	for my $eid (keys %FLmenuentry)
	{	my $menu=\@FilterList::FLcMenu;
		my $entry=$FLmenuentry{$eid};
		if (!$removeall && $::Options{'PLUGIN_'.PID.'_'.$eid})
		{	push @$menu,$entry unless (grep $_==$entry, @$menu);
		}
		else
		{	@$menu =grep $_!=$entry, @$menu;
		}
	}
}

sub Copy
{	my $IDs=$_[0]{IDs} || $_[0]{filter}->filter;
	::CopyMoveFiles($IDs,::TRUE,$::Options{'PLUGIN_'.PID.'_path'},$::Options{'PLUGIN_'.PID.'_folderformat'},$::Options{'PLUGIN_'.PID.'_filenameformat'});
}

sub ToM3U
{	my $IDs=$_[0]{IDs} || $_[0]{filter}->filter;
	my $file=::ChooseSaveFile(undef,_"Write filenames to ...",$::Songs[ $IDs->[0] ][::SONG_PATH],'list.m3u');
	return unless defined $file;
	open my$fh,'>',$file;
	for my $ID (@$IDs)
	 { print $fh $::Songs[$ID][::SONG_PATH].::SLASH.$::Songs[$ID][::SONG_FILE]."\n"; }
	close $fh;
}

sub RunCommand
{	my $IDs=$_[0]{IDs} || $_[0]{filter}->filter;
	my @cmd=split / /,$::Options{'PLUGIN_'.PID.'_tocmd_cmd'};
	return unless @cmd;
	if (grep $_ eq '$files', @cmd)
	{	my @files=map ::ReplaceFields($_,'%f'), @$IDs;
		@cmd=map { $_ ne '$files' ?  $_ :  @files } @cmd;
		::forksystem(@cmd);
	}
	else
	{	for my $ID (@$IDs)
		{	my @c=@cmd;
			$_=::ReplaceFields($ID,$_) for @c;
			system @c;
		}
	}
}

1
